package ch.psi.daq.queryrest.controller;

import java.io.IOException;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Stream;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import ch.psi.daq.cassandra.util.test.CassandraDataGen;
import ch.psi.daq.domain.cassandra.DataEvent;
import ch.psi.daq.query.model.AbstractQuery;
import ch.psi.daq.query.model.QueryField;
import ch.psi.daq.query.processor.QueryProcessor;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;

@RestController
public class QueryRestController {

   private static final Logger LOGGER = LoggerFactory.getLogger(QueryRestController.class);

   public static final String CHANNELS = "channels";
   public static final String CHANNELS_REGEX = CHANNELS + "/{regex}";
   public static final String QUERY = "query";

   @Resource
   private ResponseStreamWriter responseStreamWriter;

   @Resource
   private QueryProcessor queryProcessor;

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_FIELDS)
   private Set<QueryField> defaultResponseFields;

   @RequestMapping(
         value = CHANNELS,
         method = RequestMethod.GET,
         produces = {MediaType.APPLICATION_JSON_VALUE})
   public @ResponseBody Collection<String> getChannels() throws Throwable {
      try {
         return queryProcessor.getChannels();
      } catch (Throwable t) {
         LOGGER.error("Failed to query channel names.", t);
         throw t;
      }
   }

   @RequestMapping(
         value = CHANNELS_REGEX,
         method = RequestMethod.GET,
         produces = {MediaType.APPLICATION_JSON_VALUE})
   public @ResponseBody Collection<String> getChannels(@PathVariable String regex) throws Throwable {
      try {
         return queryProcessor.getChannels(regex);
      } catch (Throwable t) {
         LOGGER.error("Failed to query channel names with regex '{}'.", regex, t);
         throw t;
      }
   }

   @RequestMapping(
         value = QUERY,
         method = RequestMethod.GET,
         consumes = {MediaType.APPLICATION_JSON_VALUE},
         produces = {MediaType.APPLICATION_JSON_VALUE})
   public void executeQuery(@RequestBody AbstractQuery query, HttpServletResponse res) throws IOException {
      try {
         LOGGER.debug("Execute query '{}'", query.getClass().getSimpleName());

         validateQuery(query);

         // all the magic happens here
         Stream<Entry<String, Stream<? extends DataEvent>>> process = queryProcessor.process(query);

         // write the response back to the client using java 8 streams
         responseStreamWriter.respond(process, query, res);
      } catch (Throwable t) {
         LOGGER.error("Failed execute query '{}'.", query, t);
         throw t;
      }
   }

   // TODO: can we do this with built-in validators?
   private void validateQuery(AbstractQuery query) {
      if (query.getFields() == null || query.getFields().isEmpty()) {
         query.setFields(new LinkedHashSet<>(defaultResponseFields));
      }
   }

   // ==========================================================================================
   // TODO: This is simply for initial / rudimentary testing - remove once further evolved

   @Resource
   private CassandraDataGen cassandraDataGen;

   @RequestMapping(value = "/write")
   public void writeDummyEntry() {
      cassandraDataGen.writeData(3, 0, 100, "channel1", "channel2");
   }
}
