package ch.psi.daq.test.queryrest.controller;

import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import ch.psi.daq.query.model.PulseRangeQuery;
import ch.psi.daq.query.model.TimeRangeQuery;
import ch.psi.daq.query.model.TimeRangeQueryDate;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;
import ch.psi.daq.test.queryrest.query.DummyQueryProcessor;

/**
 * Tests the {@link DaqController} implementation.
 */
public class DaqRestControllerTest extends AbstractDaqRestTest {

   @Test
   public void testPulseRangeQuery() throws Exception {
      PulseRangeQuery request = new PulseRangeQuery(
            100,
            101,
            DummyQueryProcessor.TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post("/query")
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }

   @Test
   public void testTimeRangeQuery() throws Exception {
      TimeRangeQuery request = new TimeRangeQuery(
            100,
            101,
            DummyQueryProcessor.TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post("/query")
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }

   @Test
   public void testMapper() throws Exception {
      String startDate = TimeRangeQueryDate.format(100);
      String endDate = TimeRangeQueryDate.format(101);
      TimeRangeQueryDate request = new TimeRangeQueryDate(
            startDate,
            endDate,
            DummyQueryProcessor.TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post("/query")
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }
}
