/**
 * 
 */
package ch.psi.daq.test.queryrest.query;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.domain.cassandra.DataEvent;
import ch.psi.daq.query.model.Query;
import ch.psi.daq.query.processor.QueryProcessor;

/**
 * @author zellweger_c
 *
 */
public class DummyQueryProcessor implements QueryProcessor {

   public static final List<String> TEST_CHANNEL_NAMES = Lists.newArrayList("testChannel1", "testChannel2");

   @Override
   public Collection<String> getChannels() {
      return TEST_CHANNEL_NAMES;
   }

   @Override
   public Collection<String> getChannels(String regex) {
      if (regex != null) {
         Pattern pattern = Pattern.compile(regex);
         return TEST_CHANNEL_NAMES.stream().filter(channel -> pattern.matcher(channel).find())
               .collect(Collectors.toList());
      } else {
         return getChannels();
      }
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public Stream<Entry<String, Stream<? extends DataEvent>>> process(Query query) {

      int nrOfEntries = 2;
      int startingPulseId = 100;

      Map<String, Stream<? extends DataEvent>> result = Maps.newHashMap();

      TEST_CHANNEL_NAMES.forEach(chName -> {

         List<DataEvent> entities = IntStream.rangeClosed(1, nrOfEntries).boxed().map(i -> {
            long pulseId = startingPulseId + i - 1;
            return new ChannelEvent(
                  chName,
                  pulseId,
                  0,
                  pulseId,
                  pulseId,
                  0,
                  "data_" + UUID.randomUUID().toString());
         }).collect(Collectors.toList());

         result.put(chName, entities.stream());
      });

      return result.entrySet().stream();
   }
}
